/* 
 * pwm.c
 *
 * (c) 4.10.2014 Ingo D. Rullhusen
 *
 */ 

#include <linux/io.h>

#include "pwm.h"


int pwm_data_init( sunxi_pwm *PWM_DATA )
{

   PWM_DATA->name = "dcsampler/pwm";
   PWM_DATA->paddr = PWM_BASE_PTR + PWM_CTRL_REG;
   request_mem_region( PWM_DATA->paddr, 0xc, PWM_DATA->name );
   PWM_DATA->base_ptr = ioremap( PWM_BASE_PTR, 0x400 );
   if ( ! PWM_DATA->base_ptr )
   {
      return(-1);
   }

   PWM_DATA->ptr_ctrl = PWM_DATA->base_ptr + PWM_CTRL_REG;
   PWM_DATA->ptr_ch0_period = PWM_DATA->base_ptr + PWM_CH0_PERIOD;
   PWM_DATA->ptr_ch1_period = PWM_DATA->base_ptr + PWM_CH1_PERIOD;
      
      
            

   return(0);
}




void pwm_data_free( sunxi_pwm *PWM_DATA )
{

   iounmap(PWM_DATA->base_ptr);

   release_mem_region( PWM_DATA->paddr, 0xc );
      
}




void set_pwm1_rate( sunxi_pwm *PWM_DATA, unsigned int rate )
{
   unsigned long cycles;

   union
   {
      struct sun4i_pwm_ctrl clkc;
      unsigned int uiv;
   } clkc_u;

   union
   {
      struct sun4i_pwm_period clkp;
      unsigned int uiv;
   } clkp_u;

   clkc_u.uiv = readl( PWM_DATA->ptr_ctrl );
   clkp_u.uiv = readl( PWM_DATA->ptr_ch1_period );

   clkc_u.clkc.ch1_prescaler = 0xf;
   clkc_u.clkc.ch1_act_state = 1;
   clkc_u.clkc.ch1_mode = 0;
   clkc_u.clkc.ch1_pulse_start = 0;
   clkc_u.clkc.ch1_en = 0;
   clkc_u.clkc.ch1_clk_gating = 0;

   cycles = 24000000 / rate;
   clkp_u.clkp.pwm_active_cycles = cycles / 2;
   clkp_u.clkp.pwm_entire_cycles = cycles - 1;

   writel( clkp_u.uiv, PWM_DATA->ptr_ch1_period );
   writel( clkc_u.uiv, PWM_DATA->ptr_ctrl );

}



void enable_pwm1( sunxi_pwm *PWM_DATA )
{
   union
   {
      struct sun4i_pwm_ctrl clkc;
      unsigned int uiv;
   } clkc_u;

   clkc_u.uiv = readl( PWM_DATA->ptr_ctrl );

   clkc_u.clkc.ch1_en = 1;
   clkc_u.clkc.ch1_clk_gating = 1;

   writel( clkc_u.uiv, PWM_DATA->ptr_ctrl );
}

void disable_pwm1( sunxi_pwm *PWM_DATA )
{
   union
   {
      struct sun4i_pwm_ctrl clkc;
      unsigned int uiv;
   } clkc_u;

   clkc_u.uiv = readl( PWM_DATA->ptr_ctrl );

   clkc_u.clkc.ch1_en = 0;
   clkc_u.clkc.ch1_clk_gating = 0;

   writel( clkc_u.uiv, PWM_DATA->ptr_ctrl );
}

