/* 
 * pwm.h 
 *
 * (c) 4.10.2014 Ingo D. Rullhusen
 *
 */ 

#ifndef PWM_H
#define PWM_H

#include <linux/device.h>
 
 
enum sun4i_pwm_prescale { 
	PRESCALE_DIV120  = 0x00,  /* Divide 24MHz clock by 120 */ 
	PRESCALE_DIV180  = 0x01, 
	PRESCALE_DIV240  = 0x02, 
	PRESCALE_DIV360  = 0x03, 
	PRESCALE_DIV480  = 0x04, 
	PRESCALE_INVx05  = 0x05, 
	PRESCALE_INVx06  = 0x06, 
	PRESCALE_INVx07  = 0x07, 
	PRESCALE_DIV12k  = 0x08, 
	PRESCALE_DIV24k  = 0x09, 
	PRESCALE_DIV36k  = 0x0a, 
	PRESCALE_DIV48k  = 0x0b, 
	PRESCALE_DIV72k  = 0x0c, 
	PRESCALE_INVx0d  = 0x0d,
	PRESCALE_INVx0e  = 0x0e,
	PRESCALE_DIV1    = 0x0f
}; 
 
 
 
 
struct sun4i_pwm_ctrl { 
	enum sun4i_pwm_prescale ch0_prescaler:4; /* ch0 Prescale register - values above */ 
	unsigned int ch0_en:1;                  /* chan 0 enable */ 
	unsigned int ch0_act_state:1;           /* chan 0 polarity 0=low, 1=high */ 
	unsigned int ch0_clk_gating:1;          /* Allow clock to run for chan 0 */ 
	unsigned int ch0_mode:1;                /* Mode - 0 = cycle(running), 1=only 1 pulse */ 
	unsigned int ch0_pulse_start:1;         /* Write 1 for mode pulse above to start */ 
	unsigned int unused1:6;                 /* The bit skip count is 6 */ 
	enum sun4i_pwm_prescale ch1_prescaler:4; /* ch1 Prescale register - values above*/ 
	unsigned int ch1_en:1;                  /* chan 1 enable */ 
	unsigned int ch1_act_state:1;           /* chan 1 polarity 0=low, 1=high */ 
	unsigned int ch1_clk_gating:1;          /* Allow clock to run for chan 1 */ 
	unsigned int ch1_mode:1;                /* Mode - 0 = cycle(running), 1=only 1 pulse */ 
	unsigned int ch1_pulse_start:1;         /* Write 1 for mode pulse above to start */ 
	unsigned int unused2:6;                 /* The bit skip count is 6 */ 
}; 
 
 
struct sun4i_pwm_period { 
	unsigned int pwm_active_cycles:16;        /* duty cycle */ 
	unsigned int pwm_entire_cycles:16;        /* period */ 
}; 


#define PWM_BASE_PTR (0x01c20c00) 
#define PWM_CTRL_REG   (0x200)
#define PWM_CH0_PERIOD (0x204)
#define PWM_CH1_PERIOD (0x208)


typedef struct
{
   void __iomem *base_ptr;

   void __iomem *ptr_ctrl;
   void __iomem *ptr_ch0_period;
   void __iomem *ptr_ch1_period;

   phys_addr_t paddr;
   const char *name;

} sunxi_pwm;


int pwm_data_init( sunxi_pwm *PWM_DATA );
void pwm_data_free( sunxi_pwm *PWM_DATA );


void set_pwm1_rate( sunxi_pwm *PWM_DATA, unsigned int rate );
void enable_pwm1( sunxi_pwm *PWM_DATA );
void disable_pwm1( sunxi_pwm *PWM_DATA );


#endif
