/* spictl.i
 *
 * (c) 23.12.2014 Ingo D. Rullhusen
 *
 */


#define ctlregbit(rname,sname,ptr,mask) \
static ssize_t sname##_show(struct device *dev,struct device_attribute *attr, char *buf)\
{\
   ssize_t size;\
   unsigned long value;\
   sunxi_spi *data = dev_get_drvdata(dev);\
\
   printk( KERN_INFO "show " #sname "\n" );\
\
   value = readl( ptr );\
\
   size = scnprintf( buf, PAGE_SIZE, "%d", (value&mask)>0 );\
\
   return(size);\
}\
\
static ssize_t sname##_store(struct device *dev,struct device_attribute *attr, const char *buf, size_t size)\
{\
   u32 value;\
   unsigned long flags;\
   int enable = 0;\
   sunxi_spi *data = dev_get_drvdata(dev);\
\
   printk( KERN_INFO "store " #sname "\n" );\
\
   sscanf(buf,"%d",&enable);\
\
   spin_lock_irqsave(&slock, flags);\
\
   value = readl( ptr );\
\
   if ( enable ) { value |=  mask; }\
   else          { value &= ~mask; }\
\
   writel( value, ptr );\
\
   spin_unlock_irqrestore(&slock, flags);\
\
   return(size);\
}\
\
static DEVICE_ATTR(rname##_##sname, 0644, sname##_show, sname##_store)


#define ctlreghex(rname,sname,ptr,mask,bitpos) \
static ssize_t sname##_show(struct device *dev,struct device_attribute *attr, char *buf)\
{\
   ssize_t size;\
   unsigned long value;\
   sunxi_spi *data = dev_get_drvdata(dev);\
\
   printk( KERN_INFO "show " #sname "\n" );\
\
   value = readl( ptr );\
\
   size = scnprintf( buf, PAGE_SIZE, "0x%lx", (value&mask)>>bitpos );\
\
   return(size);\
}\
\
static ssize_t sname##_store(struct device *dev,struct device_attribute *attr, const char *buf, size_t size)\
{\
   u32 value;\
   unsigned long nv, flags;\
   sunxi_spi *data = dev_get_drvdata(dev);\
\
   printk( KERN_INFO "store " #sname "\n" );\
\
   sscanf(buf,"%lx",&nv);\
\
   spin_lock_irqsave(&slock, flags);\
\
   value = readl( ptr );\
\
   value &= ~mask;\
   value |= (mask&(nv<<bitpos));\
\
   writel( value, ptr );\
\
   spin_unlock_irqrestore(&slock, flags);\
\
   return(size);\
}\
\
static DEVICE_ATTR(rname##_##sname, 0644, sname##_show, sname##_store)


#define ctlreguint(rname,sname,ptr,mask,bitpos) \
static ssize_t sname##_show(struct device *dev,struct device_attribute *attr, char *buf)\
{\
   ssize_t size;\
   unsigned long value;\
   sunxi_spi *data = dev_get_drvdata(dev);\
\
   printk( KERN_INFO "show " #sname "\n" );\
\
   value = readl( ptr );\
\
   size = scnprintf( buf, PAGE_SIZE, "%lu", (value&mask)>>bitpos );\
\
   return(size);\
}\
\
static ssize_t sname##_store(struct device *dev,struct device_attribute *attr, const char *buf, size_t size)\
{\
   u32 value;\
   unsigned long nv, flags;\
   sunxi_spi *data = dev_get_drvdata(dev);\
\
   printk( KERN_INFO "store " #sname "\n" );\
\
   sscanf(buf,"%lu",&nv);\
\
   spin_lock_irqsave(&slock, flags);\
\
   value = readl( ptr );\
\
   value &= ~mask;\
   value |= (mask&(nv<<bitpos));\
\
   writel( value, ptr );\
\
   spin_unlock_irqrestore(&slock, flags);\
\
   return(size);\
}\
\
static DEVICE_ATTR(rname##_##sname, 0644, sname##_show, sname##_store)


#define ctlregbyte(rname,sname,ptr) \
static ssize_t sname##_show(struct device *dev,struct device_attribute *attr, char *buf)\
{\
   ssize_t size;\
   unsigned char value;\
   sunxi_spi *data = dev_get_drvdata(dev);\
\
   printk( KERN_INFO "show " #sname "\n" );\
\
   value = readb( ptr );\
\
   size = scnprintf( buf, PAGE_SIZE, "%i", value );\
\
   return(size);\
}\
\
static ssize_t sname##_store(struct device *dev,struct device_attribute *attr, const char *buf, size_t size)\
{\
   unsigned char value;\
   unsigned long flags;\
   sunxi_spi *data = dev_get_drvdata(dev);\
\
   printk( KERN_INFO "store " #sname "\n" );\
\
   sscanf(buf,"%i",&value);\
\
   spin_lock_irqsave(&slock, flags);\
\
   writeb( value, ptr );\
\
   spin_unlock_irqrestore(&slock, flags);\
\
   return(size);\
}\
\
static DEVICE_ATTR(rname##_##sname, 0644, sname##_show, sname##_store)



static ssize_t mtransfer_show(struct device *dev,struct device_attribute *attr, char *buf)
{
   ssize_t size;
   unsigned long value;
   sunxi_spi *data = dev_get_drvdata(dev);

   value = readl( data->ptr_rxdata );
   size = scnprintf( buf, PAGE_SIZE, "0x%08lx", value );

   return(size);
}

static ssize_t mtransfer_store(struct device *dev,struct device_attribute *attr, const char *buf, size_t size)
{
   u32 value;
   unsigned long flags;
   sunxi_spi *data = dev_get_drvdata(dev);

   sscanf(buf,"%x",&value);

   spin_lock_irqsave(&slock, flags);

   writel( value, data->ptr_txdata );

   writel( 0x00000004, data->ptr_tc );
   writel( 0x00000004, data->ptr_bc );
   
   value = readl( data->ptr_ctl ) | SPI_CTL_XCH;
   writel( value, data->ptr_ctl );

   spin_unlock_irqrestore(&slock, flags);

   return(size);
}

static DEVICE_ATTR(mtransfer, 0644, mtransfer_show, mtransfer_store);



//
// SPI_RXDATA

ctlreghex( rx, rdata, data->ptr_rxdata, 0xffffffff, 0 );
//ctlregbyte( rx, rdata, data->ptr_rxdata );

//
// SPI_TXDATA

ctlreghex( tx, tdata, data->ptr_txdata, 0xffffffff, 0 );
//ctlregbyte( tx, tdata, data->ptr_txdata );

//
// SPI_CTL

ctlregbit( ctl, sdm, data->ptr_ctl, SPI_CTL_MASTER_SDM );
ctlregbit( ctl, sdc, data->ptr_ctl, SPI_CTL_MASTER_SDC );
ctlregbit( ctl, tp_en, data->ptr_ctl, SPI_CTL_T_PAUSE_EN );
ctlregbit( ctl, ss_level, data->ptr_ctl, SPI_CTL_SS_LEVEL );
ctlregbit( ctl, ss_ctl, data->ptr_ctl, SPI_CTL_SS_CTRL );
ctlregbit( ctl, dhb, data->ptr_ctl, SPI_CTL_DHB );
ctlregbit( ctl, ddb, data->ptr_ctl, SPI_CTL_DDB );
ctlreguint( ctl, ss, data->ptr_ctl, SPI_CTL_SS_MASK, SPI_SS_BIT_POS );
ctlregbit( ctl, rpsm, data->ptr_ctl, SPI_CTL_RAPIDS );
ctlregbit( ctl, xch, data->ptr_ctl, SPI_CTL_XCH );
ctlregbit( ctl, rf_rst, data->ptr_ctl, SPI_CTL_RST_RXFIFO );
ctlregbit( ctl, tf_rst, data->ptr_ctl, SPI_CTL_RST_TXFIFO );
ctlregbit( ctl, ssctl, data->ptr_ctl, SPI_CTL_SSCTL );
ctlregbit( ctl, lmtf, data->ptr_ctl, SPI_CTL_LMTF );
ctlregbit( ctl, dmamc, data->ptr_ctl, SPI_CTL_DMAMOD );
ctlregbit( ctl, sspol, data->ptr_ctl, SPI_CTL_SSPOL );
ctlregbit( ctl, pol, data->ptr_ctl, SPI_CTL_POL );
ctlregbit( ctl, pha, data->ptr_ctl, SPI_CTL_PHA );
ctlregbit( ctl, mode, data->ptr_ctl, SPI_CTL_FUNC_MODE );
ctlregbit( ctl, enable, data->ptr_ctl, SPI_CTL_EN );

//
// SPI_INTCTL

ctlregbit( intctl, ss_int_en, data->ptr_intctl, SPI_INTEN_SSI );
ctlregbit( intctl, tx_int_en, data->ptr_intctl, SPI_INTEN_TC );
ctlregbit( intctl, tf_ur_int_en, data->ptr_intctl, SPI_INTEN_TU );
ctlregbit( intctl, tf_of_int_en, data->ptr_intctl, SPI_INTEN_TO );
ctlregbit( intctl, tf_e34_int_en, data->ptr_intctl, SPI_INTEN_3QTR_TE );
ctlregbit( intctl, tf_e14_int_en, data->ptr_intctl, SPI_INTEN_QTR_TE );
ctlregbit( intctl, tf_fl_int_en, data->ptr_intctl, SPI_INTEN_TF );
ctlregbit( intctl, tf_half_emp_int_en, data->ptr_intctl, SPI_INTEN_TH );
ctlregbit( intctl, tx_emp_int_en, data->ptr_intctl, SPI_INTEN_TE );
ctlregbit( intctl, rf_ur_int_en, data->ptr_intctl, SPI_INTEN_RU );
ctlregbit( intctl, rf_of_int_en, data->ptr_intctl, SPI_INTEN_RO );
ctlregbit( intctl, rf_f34_int_en, data->ptr_intctl, SPI_INTEN_3QTR_RF );
ctlregbit( intctl, rf_f14_int_en, data->ptr_intctl, SPI_INTEN_QTR_RF );
ctlregbit( intctl, rf_fu_int_en, data->ptr_intctl, SPI_INTEN_RF );
ctlregbit( intctl, rf_half_fu_int_en, data->ptr_intctl, SPI_INTEN_RH );
ctlregbit( intctl, rf_rdy_int_en, data->ptr_intctl, SPI_INTEN_RR );

//
// SPI_INT_STA

ctlregbit( int_sta, ssi, data->ptr_int_sta, SPI_STAT_SSI );
ctlregbit( int_sta, tc, data->ptr_int_sta, SPI_STAT_TC );
ctlregbit( int_sta, tu, data->ptr_int_sta, SPI_STAT_TU );
ctlregbit( int_sta, to, data->ptr_int_sta, SPI_STAT_TO );
ctlregbit( int_sta, te34, data->ptr_int_sta, SPI_STAT_3QTR_TE );
ctlregbit( int_sta, te14, data->ptr_int_sta, SPI_STAT_QTR_TE );
ctlregbit( int_sta, tf, data->ptr_int_sta, SPI_STAT_TF );
ctlregbit( int_sta, the, data->ptr_int_sta, SPI_STAT_THE );
ctlregbit( int_sta, te, data->ptr_int_sta, SPI_STAT_TE );
ctlregbit( int_sta, ru, data->ptr_int_sta, SPI_STAT_RU );
ctlregbit( int_sta, ro, data->ptr_int_sta, SPI_STAT_RO );
ctlregbit( int_sta, rf34, data->ptr_int_sta, SPI_STAT_3QTR_RF );
ctlregbit( int_sta, rf14, data->ptr_int_sta, SPI_STAT_QTR_RF );
ctlregbit( int_sta, rf, data->ptr_int_sta, SPI_STAT_RF );
ctlregbit( int_sta, rhf, data->ptr_int_sta, SPI_STAT_RHF );
ctlregbit( int_sta, rr, data->ptr_int_sta, SPI_STAT_RR );

//
// SPI_DMA

ctlregbit( dmactl, tf_emp34, data->ptr_dmactl, SPI_DRQEN_3QTR_TE );
ctlregbit( dmactl, tf_emp14, data->ptr_dmactl, SPI_DRQEN_QTR_TE );
ctlregbit( dmactl, tf_nf, data->ptr_dmactl, SPI_DRQEN_TNF );
ctlregbit( dmactl, tf_he, data->ptr_dmactl, SPI_DRQEN_THE );
ctlregbit( dmactl, tf_emp, data->ptr_dmactl, SPI_DRQEN_TE );
ctlregbit( dmactl, rf_fu34, data->ptr_dmactl, SPI_DRQEN_3QTR_RF );
ctlregbit( dmactl, rf_fu14, data->ptr_dmactl, SPI_DRQEN_QTR_RF );
ctlregbit( dmactl, rf_fu, data->ptr_dmactl, SPI_DRQEN_RF );
ctlregbit( dmactl, rf_hf, data->ptr_dmactl, SPI_DRQEN_RHF );
ctlregbit( dmactl, rf_rdy, data->ptr_dmactl, SPI_DRQEN_RR );

//
// SPI_WAIT

ctlreguint( wait, wcc, data->ptr_wait, SPI_WAIT_CLK_MASK, 0 );

//
// SPI_CCTL

ctlregbit( cctl, drs, data->ptr_cctl, SPI_CLKCTL_DRS );
ctlreguint( cctl, cdr1, data->ptr_cctl, SPI_CLKCTL_CDR1, 8 );
ctlreguint( cctl, cdr2, data->ptr_cctl, SPI_CLKCTL_CDR2, 0 );

//
// SPI_BC

ctlreguint( bc, bc, data->ptr_bc, SPI_BC_BC_MASK, 0 );

//
// SPI_TC

ctlreguint( tc, wtc, data->ptr_tc, SPI_TC_WTC_MASK, 0 );

//
// SPI_FIFO_STA

ctlreguint( fifo_sta, tf_cnt, data->ptr_fifo_sta, SPI_FIFO_TXCNT, SPI_TXCNT_BIT_POS );
ctlreguint( fifo_sta, rf_cnt, data->ptr_fifo_sta, SPI_FIFO_RXCNT, SPI_RXCNT_BIT_POS );



static const struct attribute *spi_attrs[] =
{
   &dev_attr_rx_rdata.attr,
   &dev_attr_tx_tdata.attr,

   &dev_attr_ctl_sdm.attr,
   &dev_attr_ctl_sdc.attr,
   &dev_attr_ctl_tp_en.attr,
   &dev_attr_ctl_ss_level.attr,
   &dev_attr_ctl_ss_ctl.attr,
   &dev_attr_ctl_dhb.attr,
   &dev_attr_ctl_ddb.attr,
   &dev_attr_ctl_ss.attr,
   &dev_attr_ctl_rpsm.attr,
   &dev_attr_ctl_xch.attr,
   &dev_attr_ctl_rf_rst.attr,
   &dev_attr_ctl_tf_rst.attr,
   &dev_attr_ctl_ssctl.attr,
   &dev_attr_ctl_lmtf.attr,
   &dev_attr_ctl_dmamc.attr,
   &dev_attr_ctl_sspol.attr,
   &dev_attr_ctl_pol.attr,
   &dev_attr_ctl_pha.attr,
   &dev_attr_ctl_mode.attr,
   &dev_attr_ctl_enable.attr,

   &dev_attr_intctl_ss_int_en.attr,
   &dev_attr_intctl_tx_int_en.attr,
   &dev_attr_intctl_tf_ur_int_en.attr,
   &dev_attr_intctl_tf_of_int_en.attr,
   &dev_attr_intctl_tf_e34_int_en.attr,
   &dev_attr_intctl_tf_e14_int_en.attr,
   &dev_attr_intctl_tf_fl_int_en.attr,
   &dev_attr_intctl_tf_half_emp_int_en.attr,
   &dev_attr_intctl_tx_emp_int_en.attr,
   &dev_attr_intctl_rf_ur_int_en.attr,
   &dev_attr_intctl_rf_of_int_en.attr,
   &dev_attr_intctl_rf_f34_int_en.attr,
   &dev_attr_intctl_rf_f14_int_en.attr,
   &dev_attr_intctl_rf_fu_int_en.attr,
   &dev_attr_intctl_rf_half_fu_int_en.attr,
   &dev_attr_intctl_rf_rdy_int_en.attr,

   &dev_attr_int_sta_ssi.attr,
   &dev_attr_int_sta_tc.attr,
   &dev_attr_int_sta_tu.attr,
   &dev_attr_int_sta_to.attr,
   &dev_attr_int_sta_te34.attr,
   &dev_attr_int_sta_te14.attr,
   &dev_attr_int_sta_tf.attr,
   &dev_attr_int_sta_the.attr,
   &dev_attr_int_sta_te.attr,
   &dev_attr_int_sta_ru.attr,
   &dev_attr_int_sta_ro.attr,
   &dev_attr_int_sta_rf34.attr,
   &dev_attr_int_sta_rf14.attr,
   &dev_attr_int_sta_rf.attr,
   &dev_attr_int_sta_rhf.attr,
   &dev_attr_int_sta_rr.attr,

   &dev_attr_dmactl_tf_emp34.attr,
   &dev_attr_dmactl_tf_emp14.attr,
   &dev_attr_dmactl_tf_nf.attr,
   &dev_attr_dmactl_tf_he.attr,
   &dev_attr_dmactl_tf_emp.attr,
   &dev_attr_dmactl_rf_fu34.attr,
   &dev_attr_dmactl_rf_fu14.attr,
   &dev_attr_dmactl_rf_fu.attr,
   &dev_attr_dmactl_rf_hf.attr,
   &dev_attr_dmactl_rf_rdy.attr,

   &dev_attr_wait_wcc.attr,

   &dev_attr_cctl_drs.attr,
   &dev_attr_cctl_cdr1.attr,
   &dev_attr_cctl_cdr2.attr,

   &dev_attr_bc_bc.attr,

   &dev_attr_tc_wtc.attr,
   
   &dev_attr_fifo_sta_tf_cnt.attr,
   &dev_attr_fifo_sta_rf_cnt.attr,


   &dev_attr_mtransfer.attr,
   NULL
};

static const struct attribute_group spi_attr_group =
{
   .attrs = (struct attribute **) spi_attrs
};


static char *spi_name[] = { "spi0", "spi1", "spi2", "spi3" };


